/****************************************************************************
*
*	  Header Filename:  Realisable_Interaction_Type.h
*	  Module Name:      Realisable_Interaction_Type
*	  Description:      An object of this class represents an Interaction_Type
*                       which is currently realisable in the Reactor.
*                       That is, if a particular Realisable_Interaction_Type exists,
*                       the Reactant(s) and any Catalyst required
*                       for the Interaction to occur are currently present
*                       in the Reactor.
*	  Author:   		Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

#ifndef Realisable_Interaction_Type_H
#define Realisable_Interaction_Type_H

class   Interaction_Type;
class   Realised_Interaction_Type;

class   Realisable_Interaction_Type

//  An object of this class represents an Interaction_Type
//  which is currently realisable in the Reactor.
//  That is, if a particular Realisable_Interaction_Type exists,
//  the Reactant(s) and any Catalyst required
//  for the Interaction to occur are currently present
//  in the Reactor.

{
    friend class Reactor;

    private:
//===== Relationships with other classes ===================================================================

        Interaction_Type*   Interaction_Type__Ptr; // Link to the Interaction_Type
        DN::Node<Realisable_Interaction_Type*>*   Node__In_Reactors_List_Ptr;
                                             // Pointer to this Realisable_Interaction_Type's
                                             // Node in the Reactor's List of Realisable_Int_Types
        Realised_Interaction_Type*  Realised_Int_Ptr;

        DN::List<DN::Node<Realisable_Interaction_Type*>* >  List__Nodes_In_R_Mol_Types_Lists;

                                            // List of Nodes for this Realisable_Interaction_Type in
                                            // the Realised_Molecule_Type's Lists to which
                                            // it is linked. This list can be used to rapidly unlink
                                            // the Realisable_Interaction_Type from the
                                            // Realised_Molecule_Types to which it is linked.
//==========================================================================================================

    public:
//      Public Functions

        void  Disable() {Enabled = false;};
        void  Enable()  {Enabled = true;};
        bool  Get_Unrealisable() const;
        void  Set_Unrealisable();
        double Get_Interaction_Rate() const;
        Interaction_Type*   const Get_Interaction_Type_Ptr() const;
        Realisable_Interaction_Type (Interaction_Type*  Interaction_Type__Ptr);
        double Update_Interaction_Rates(int Reactor_Volume);
        DN::Node<Realisable_Interaction_Type*>*  Get_Node_In_Reactors_List_Ptr() const;
        void  Set_Node_In_Reactors_List_Ptr(DN::Node<Realisable_Interaction_Type*>*  Node__Ptr);
        
    private:
//      Private Class Data

        double Interaction_Rate;  //  Rate at which Interaction Type occurs in the Reactor
        bool  Unrealisable;       //  Set true if this Realisable_Interaction_Type
                                  //  became unrealisable in the Reactor because the required
                                  //  Reactant(s) are not present
        bool  Enabled;            //  Set false  if the Realisable_Interaction_Type is disabled.
                                  //  eg as part of a mutation
       
                                
//      Private Functions

};

//	Declaration of Data Structures constructed by this class only

//  None

//	Defines used with class interface (always prefixed with class name)

//	None


#endif  //Realisable_Interaction_Type_H
