/****************************************************************************
*
*	  Header Filename:  Attribute.h
*	  Module Name:      Attribute
*	  Description:      Helper classes to support Simulation Attributes
*
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*      Amendment History:- 
*
*              Version	  Date                        Update Description
*              =======    ===========                 ==================
*
*
*
*
****************************************************************************/

#ifndef Attribute_H
#define Attribute_H

namespace Attribute
{
    enum AttrCode
        {
            ActReq_Type_Cd,        // Action Request Type
            ACE_Cd,                // Activated Complex Energy Spec
            Num_To_Add_Cd,
            Mass_Cd,
            MolType_Cd,
            Pot_Energy_Cd,
            Product_Cd,
            Product1_Cd,
            Product2_Cd,
            Reactant_Cd,
            Reactant1_Cd,
            Reactant2_Cd,
            Repeat_Spec_Cd,
            Time_Cd
        };

        class  AttrVal  // Helper class for attribute values
        {
            public:
//              Text signifying attribute is to be given a random value
                static  string  Get_Rnd_Tx() {return string("Random");}  
        };
        
};   // End of namespace Attribute

class   AttrNames

//  Attribute Names manager. Includes a mapping of names to codes

{
    public:
        AttrNames();
        static  void  Make();
        static  string   Cd_To_Name(Attribute::AttrCode  Code);

    private:
        map<Attribute::AttrCode, string>  Attributes;
        static AttrNames*  Instance_Ptr;       

        AttrNames(const AttrNames&);
        AttrNames& operator= (const AttrNames&);  
};
//----------------------------------------------------------------------------------------------------------------

class ACE_Spec{

//  Activated Complex Energy Specifier

    public:
        ACE_Spec(bool Rnd_Fg, double Energy);
        ACE_Spec(const ACE_Spec& Existing);
        
        bool    Get_Rnd_Fl() const {return   Rnd_Fg_;};
        double  Get_Energy()  const {return   ACE_;};        
        string  Get_Spec_As_Tx() const 
                {return (Rnd_Fg_ ? Attribute::AttrVal::Get_Rnd_Tx() : To_String(ACE_));}

    private:
        const   bool    Rnd_Fg_;     // Whether Random
        const   double  ACE_;        // Energy value. Only used if Random == false

        ACE_Spec& operator= (const ACE_Spec&);  
};

//----------------------------------------------------------------------------------------------------------------

class MolType_Spec{

//  Molecule Type Specifier

    public:
        MolType_Spec(bool Rand_Flag, int MolType_Id);
        MolType_Spec(const MolType_Spec&  Existing);

        bool    Get_Rnd_Fl() const {return Rand_Flag_;};
        int     Get_MolType_Id() const {return MolType_Id_;};
        string  Get_Spec_As_Text() const 
            {return (Rand_Flag_ ? Attribute::AttrVal::Get_Rnd_Tx()  : To_String(MolType_Id_));};

    private:
        const bool    Rand_Flag_;    // Whether Random
        const int     MolType_Id_;    // Only used if Random == false

        MolType_Spec& operator= (const MolType_Spec&); 
};
//----------------------------------------------------------------------------------------------------------------

class Mass_Spec{

//  Mass Specifier

    public:
        Mass_Spec(bool Rand_Flag, int Mass);
        Mass_Spec(const Mass_Spec& Existing);

        bool    Get_Rand_Flag() const {return Rand_Flag_;};
        int     Get_Mass() const  {return  Mass_;};
        string  Get_Spec_As_Text() 
            {return (Rand_Flag_ ? Attribute::AttrVal::Get_Rnd_Tx() : To_String(Mass_));}

    private:
        const bool    Rand_Flag_;      // Whether Random
        const int     Mass_;           // Only used if Random == false

        Mass_Spec& operator= (const Mass_Spec&);  
};
//----------------------------------------------------------------------------------------------------------------
        
class PotEnergy_Spec{

//  Potential Energy Specifier

    public:
        PotEnergy_Spec(bool Rand_Flag, double Pot_Energy);
        PotEnergy_Spec(const PotEnergy_Spec& Existing);
        
        bool    Get_Rand_Flag() const {return   Rand_Flag_;};
        double  Get_Pot_Energy()  const {return   PotEnergy_;};        
        string  Get_Spec_As_Text() const 
            {return (Rand_Flag_ ? Attribute::AttrVal::Get_Rnd_Tx() : To_String(PotEnergy_));}

    private:
        const   bool    Rand_Flag_;    // Whether Random
        const   double  PotEnergy_;    // Only used if Random == false

        PotEnergy_Spec& operator= (const PotEnergy_Spec&);  
};

#endif  //Attribute_H

