/****************************************************************************
*
*	  Header Filename:  Indexed_Series.h
*	  Module Name:      Indexed_Series
*	  Description:      Stores and maintains a series of values 
*                       that are accessible by an Index
*                       The Index and teh Value together can be used to represent a data pair
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*      Amendment History:- 
*
*              Version	  Date                        Update Description
*              =======    ===========                 ==================
*
*
*
*
****************************************************************************/

#ifndef Indexed_Series_H
#define Indexed_Series_H

class   Indexed_Series

{
//===== Relationships with other classes ===================================================================
    
//==========================================================================================================

    public:
        void    Add_Value(double Index, double Value);
        bool    Axes_Are_Changed();
        static bool Axis_Is_Changed(double Old_Range_Min, double Old_Range_Max,double New_Range_Min, double New_Range_Max);
        bool    No_Entry_Means_Zero();
        void    Clear();
        Indexed_Series (string  Indexed_Series_Name, 
                        string Series_List,
                        string X_Coord_Name, 
                        string Y_Coord_Name, 
                        bool No_Entry_Means_Zero);
        bool    Get_Value_For_Index(double& Index, double& Value);
        bool    Get_First_Data_Pair(double& Index, double& Value);
        bool    Get_Next_Data_Pair(double& Index, double& Value);
        string  Get_Indexed_Series_Name() const;
        string  Get_Series_List_Name();
        void    Get_Series_Ranges(double& Index_Min,double& Index_Max,
                                  double& Value_Min,double& Value_Max);
        string  Get_Index_Name(){return Index_Name_;};
        string  Get_Value_Name(){return Value_Name_;};
        void    Get_Index_Axis(double& Index_First_Tick, double& Index_Last_Tick, double& Index_Increment);
        void    Get_Value_Axis(double& Value_First_Tick, double& Value_Last_Tick, double& Value_Increment);
        void    Toggle_Series_Display();
        int     Get_Num_Values();
        bool    Is_For_Display();
        static void   Ticks_And_Increment_For_Range(double Range_Min, double Range_Max, 
                                                    double& First_Tick, double& Last_Tick, double& Increment);                                                      

        void    Update_Axes();

    private:      

        void   Init_Ranges_And_Axes();
        string Indexed_Series_Name_;
        string Series_List_Name_;
        string Index_Name_;
        string Value_Name_;
        map <double,double>  Value_List;
        map <double,double>::const_iterator Value_List_Iter;
        map <double,double>::iterator Last_Inserted_Pos;
        bool   Display_Flag;   // Set true if Series is to be displayed
        bool   Treat_No_Entry_As_Zero;        // Set true if a missing entry for an index is to 
                                              // be interpreted as zero.
//        bool   User_Visibl_Fl_;   // If false, then the series is not intended for user display
 
        struct Series_Ranges 
        {
            double Index_Min;
            double Index_Max;
            double Value_Min;
            double Value_Max;
        }; 
        Series_Ranges  Ranges;
        
        struct Axis
        {
            double First_Tick;
            double Last_Tick;
            double Increment;
        };   
        Axis Previous_Index_Axis;  
        Axis Previous_Value_Axis;
        Axis Current_Index_Axis;  
        Axis Current_Value_Axis;
};
#endif  //Indexed_Series_H
