/****************************************************************************
*
*	  Module Filename:  Form_Simulation_Statistics.cpp
*	  Module Name:      Form_Simulation_Statistics
*	  Description:      Manages user interface for display of Simulation Statistics                   
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*User_Actions_API::Form_SimStats::ListView_Series_DblClick
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/
//  Comment out to disallow tracing this file
//#define  ENABLE_TRACE_FILE  

#include    "globldef.h"
#include    "Library_Headers.h"

//--------------  GUI Presentation Subsystem  --------------------------------------------------------------------
#include  "GUI_Presentation_API.h"
//-----------------------------------------------------------------------------------------------------------------

//----------------  User Interface Manager  ----------------------------------------------------------------------
#include    "Form_Simulation_Statistics.h"
#include    "Form_Main.h"
//---------------------------------------------------------------------------------------------------------------

//----------------Statistics Subsystem  --------------------------------------------------------------------------
#include    "Simulation_Statistics.h"
#include    "Indexed_Series.h"
//---------------------------------------------------------------------------------------------------------------

//--------------  Network Painter Subsystem  ---------------------------------------------------------------------
#include  "Network_Painter_Prot.h"
//--------------------------------------------------------------------------------------------------------------

#include  "Krnl_Vis.h"

Form_Simulation_Statistics* Form_Simulation_Statistics::Instance_Ptr = 0;

//==============================================================================================================##

 void    Form_Simulation_Statistics::Clear()

//  Clear the Form

{
    TRACE("Form_Simulation_Statistics::Clear");
    Init_Series_Display();

    Current_Display_Group = Series_Plots;
    Last_Display_Time = -1;  // Means never displayed

    GUI_Presentation_API::Clear(Widget_Ids::SimStats_SerList_ListStore);
    GUI_Presentation_API::Clear(Widget_Ids::SimStats_Series_ListStore);
    GUI_Presentation_API::Clear(Widget_Ids::SimStats_Groups_ListStore);
    GUI_Presentation_API::Clear(Widget_Ids::SimStats_Draw_Area);
    GUI_Presentation_API::Draw_Pixmap(Widget_Ids::SimStats_Draw_Area);

    List_Groups();
    Curr_SerList_Name_Tx_ = SimStats_Ptr->Get_Default_SerList_Nm();
    List_Series_Lists();
    SimStats_Ptr->Set_Current_Ind_Series_List(Curr_SerList_Name_Tx_);
    List_Series();
}
//----------------------------------------------------------------------------------------------------------------

bool Form_Simulation_Statistics::Compare_Indexed_Series_Ranges (pair <Indexed_Series*, double> Pair1, pair <Indexed_Series*, double> Pair2)

//  Compare Series Ranges. Return true if the first series has the larger range.

{
    double  P1_Index_Min;
    double  P1_Index_Max;
    double  P1_Value_Min;
    double  P1_Value_Max;   
    double  P2_Index_Min;
    double  P2_Index_Max;
    double  P2_Value_Min;
    double  P2_Value_Max;                                                
                                     
    Pair1.first->Get_Series_Ranges(P1_Index_Min, P1_Index_Max, P1_Value_Min, P1_Value_Max);
    Pair2.first->Get_Series_Ranges(P2_Index_Min, P2_Index_Max, P2_Value_Min, P2_Value_Max);
    
    if ((P1_Value_Max - P1_Value_Min) > (P2_Value_Max - P2_Value_Min))
    THEN
        return true;
    ELSE
        return false;
    ENDIF
} 
//-----------------------------------------------------------------------------------------------------------------


void    Form_Simulation_Statistics::Display_Series_Name(Indexed_Series* Indexed_Series_Ptr)

//  Display the Indexed Series Name and Link the item to the Data series

{
    TRACE("Form_Simulation_Statistics::Display_Series_Name");
    list<string> List_Of_Strings;
    List_Of_Strings.push_back(Indexed_Series_Ptr->Get_Indexed_Series_Name());

    string Plot_Text;
    if (Indexed_Series_Ptr->Is_For_Display())
    THEN
        Plot_Text = "Yes";
    ELSE
        Plot_Text = "No";
    ENDIF

    List_Of_Strings.push_back(Plot_Text);
    GUI_Presentation_API::Add_Row(Widget_Ids::SimStats_Series_ListStore, 
                                  List_Of_Strings, 
                                  Indexed_Series_Ptr); // Link the item to the Data series
}
//----------------------------------------------------------------------------------------------------------------
        
void    Form_Simulation_Statistics::Display_Series_Plots(Display_Option Display_Opt)

//  Display the Indexed Series Plots. Only display those Series that have been
//  marked for plotting

//  If Display_Opt == optimise, then optimise the display by suppressing painting items 
//  that don't need to be repainted, 

{
    TRACE("Form_Simulation_Statistics::Display_Series_Plots");
        
//  Set Plot Area (need to do it here in case form has been resized)

    int Width;
    int Height;
    GUI_Presentation_API::Get_Size(Widget_Ids::SimStats_Draw_Area, Width, Height);

    SimStats_Plot_Area.Left = 50;
    SimStats_Plot_Area.Right = Width - 100;

    int Num_Series = Num_Series_For_Display();
    if (Num_Series > 2)
    THEN 
//      Adjust to make way for right axis
        SimStats_Plot_Area.Right -= (Num_Series-2) * 30;
    ENDIF
                                                                         
    SimStats_Plot_Area.Top = 50;
    SimStats_Plot_Area.Bottom = Height - 55;

    double Index_Min = DBL_MAX;
    double Index_Max = -DBL_MAX;
    double First_Tick, Last_Tick, Increment;
    
//  Update the axes for each plot. Determine whether there is a change of axis required for any plot. 
//  If there is a change of axis for any plot, or if 'Full' was specified in the argument, then 
//  a full repaint is done.

    bool  Full_Repaint_Flag;
    if (Display_Opt == Full)
    THEN
        Full_Repaint_Flag = true;
    ELSE
        Full_Repaint_Flag = false; 
    ENDIF
    
    list <Series_And_Colour_To_Plot>::iterator  Iter;
    for (Iter = List_Of_Series_To_Plot.begin();
         Iter not_eq List_Of_Series_To_Plot.end();
         Iter++)
    BEGIN
        Indexed_Series*  Series_Ptr = (*Iter).Series_Ptr;    
        if (Series_Ptr->Get_Num_Values() > 1)
        THEN
            Series_Ptr->Update_Axes(); 
            if (Series_Ptr->Axes_Are_Changed());
            THEN
                Full_Repaint_Flag = true;
            ENDIF
        ENDIF
    END

//  If a full replot is needed then clear the Drawing Area, otherwise just add to it    
        
    if (Full_Repaint_Flag)
    THEN
        GUI_Presentation_API::Clear(Widget_Ids::SimStats_Draw_Area); 
    ENDIF
            
//  Plot the Series that are to be displayed, excluding any with less than two data points

    unsigned int Display_Series_Counter = 0;              
    for (Iter = List_Of_Series_To_Plot.begin();
         Iter not_eq List_Of_Series_To_Plot.end();
         Iter++)
    BEGIN
        Indexed_Series*  Series_Ptr = (*Iter).Series_Ptr;
        Colour_Id Plot_Colour = (*Iter).Plot_Colour;
        if (Series_Ptr->Get_Num_Values() > 1)
        THEN
            Display_Series_Counter++;
            Plot_Indexed_Series(Series_Ptr, 
                                Plot_Colour, 
                                true,   // Gives a better display, but a little slower
                                Krnl_Vis::Reactor_::Get_Time());
         //   Plot_Series(Series_Ptr, Plot_Colour, Full_Repaint_Flag);   
            
            if (Full_Repaint_Flag)
            THEN
                Draw_Series_Y_Axis(Series_Ptr, Plot_Colour, string(""), Display_Series_Counter);
            ENDIF          
      
//          Get min / max values for all series that will share the X axis 
//          ***** NEED A MORE GENERIC SOLUTION. FOR NOW USE AN ASSERTION TO CHECK FOR ANY PROBLEM
            
            double Index_Min_Series, Index_Max_Series;
            double Value_Min_Series, Value_Max_Series;
            Series_Ptr->Get_Series_Ranges(Index_Min_Series, Index_Max_Series, Value_Min_Series, Value_Max_Series);
             
            if (Display_Series_Counter > 1)
            THEN
                Assert((Index_Min == Index_Min_Series) and (Index_Max == Index_Max_Series), 
                       "Form_Simulation_Statistics::Display_Statistics - Differences in ranges for Index Values in different series");
 //                      + string("Index_Min_Series = ") + To_String(Index_Min_Series) + string(" Index_Max_Series = ") + To_String(Index_Max_Series)
//                       + string(" Index_Min = ") + To_String(Index_Min) + string(" Index_Max = ") + To_String(Index_Max)).c_str());
            ENDIF
            
//          Now ouput the name of the series
    
            int Series_Name_Label_X =  SimStats_Plot_Area.Left + 50;
            int Series_Name_Label_Y =  SimStats_Plot_Area.Top + (Display_Series_Counter - 3)* 20;   


            GUI_Presentation_API::Text_Out(Widget_Ids::SimStats_Draw_Area, 
                                           Series_Name_Label_X,
                                           Series_Name_Label_Y,
                                           Series_Ptr->Get_Value_Name());
                
            Index_Min = min(Index_Min_Series, Index_Min);
            Index_Max = max(Index_Max_Series, Index_Max);
        ENDIF
    END
    
    if (Display_Series_Counter not_eq 0)
    THEN
        Indexed_Series::Ticks_And_Increment_For_Range(Index_Min, Index_Max, First_Tick, Last_Tick, Increment);
           
        if (Full_Repaint_Flag)
        THEN
            Draw_X_Axis(First_Tick, Last_Tick, Increment, Colour_Ids::Black, string("Time"));
        ENDIF          
    ENDIF
} 
//---------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::Display_Manhattan_Plot(Display_Option Display_Opt, bool Simulation_Running)

//  Display a plot of Manhattan Distances between Composome at "Base Time"
//  and Composome at "Time"

//  Optimise display or Full Repaint depending on the argument and whether there is a change of axis

{
//  Setup the plot area

    int Width;
    int Height;
    GUI_Presentation_API::Get_Size(Widget_Ids::SimStats_Draw_Area, Width, Height);

    SimStats_Plot_Area.Left = 50;
    SimStats_Plot_Area.Right = Width - 50;
                                                               
    SimStats_Plot_Area.Top = 50;
    SimStats_Plot_Area.Bottom = Height - 50;

    int Sensitivity_Text_X =  SimStats_Plot_Area.Left + 200;
    int Sensitivity_Text_Y =  SimStats_Plot_Area.Bottom - 150;   
    int Plotting_Text_X = Sensitivity_Text_X;
    int Plotting_Text_Y = Sensitivity_Text_Y + 50;

//  Select the List of Indexed_Series to be used to be displayed as a Manhattan Plot

    Simulation_Statistics*  SimStats_Ptr = Simulation_Statistics::Get_Ptr();    
    SimStats_Ptr->Set_Current_Ind_Series_List(string("Manhattan Distances"));
    
    Indexed_Series*  Last_Series_Ptr;
    double First_Tick, Last_Tick, Increment;      
    bool Full_Repaint_Flag = false;  // May be changed later if axes need to be redrawn
    
    if(Display_Opt == Full)
    THEN
        Full_Repaint_Flag = true;
    ENDIF
    
    bool Display_Required = true;  // Will be set false if no data to display

//  Determine whether a Full Repaint is required. If there is a change of axis
//  required (based on the Index of the last and penultimate Indexed Series),
//  then set the Full Repaint flag.

//  If there is no data, then set the Display Required flag accordingly.
    
    if (SimStats_Ptr->Iter_To_Last_Ind_Series())
    THEN
        Last_Series_Ptr =  SimStats_Ptr->Get_Current_Indexed_Series_Ptr();
        if (Last_Series_Ptr->Get_Num_Values() > 0)
        THEN
            double Dummy1;
            double Last_Series_Index_Min, Last_Series_Index_Max;
            Last_Series_Ptr->Update_Axes();
            Last_Series_Ptr->Get_Series_Ranges(Last_Series_Index_Min, Last_Series_Index_Max, Dummy1, Dummy1);  
            
            if (SimStats_Ptr->Iter_To_Prev_Ind_Series())
            THEN
                Indexed_Series*  Penultimate_Series_Ptr;
                Penultimate_Series_Ptr =  SimStats_Ptr->Get_Current_Indexed_Series_Ptr();
                double Dummy2;
                double Penultimate_Series_Index_Min, Penultimate_Series_Index_Max;
                Penultimate_Series_Ptr->Get_Series_Ranges(Penultimate_Series_Index_Min, Penultimate_Series_Index_Max, Dummy2, Dummy2);
                if (Indexed_Series::Axis_Is_Changed(Penultimate_Series_Index_Min, Penultimate_Series_Index_Max,
                                                    Last_Series_Index_Min, Last_Series_Index_Max))
                THEN
                    Full_Repaint_Flag = true;
                ENDIF
            ENDIF
        ELSE
            Display_Required = false;   // No values in the series
        ENDIF
    ELSE
        Display_Required = false;       // No series
    ENDIF
    
//  Get the current Number of Molecules in the Reactor. This
//  is used to derive the current maximum Manhattan Distance
//  REMOVE THIS LATER. IT WILL BE BETTER TO STORE THIS 
//  MAXIMUM LENGTH AS PART OF THE MANHATTAN DISTANCE SERIES

    if (Display_Required)
    THEN
        int Current_Num_Mols = Krnl_Vis::Reactor_::Get_Num_Mols();
        
//      Get the start and end points for the axes
//      Both axes are based on the Index (which corresponds to Time) of the last Indexed Series
        
        Last_Series_Ptr->Get_Index_Axis(First_Tick, Last_Tick, Increment);
        
        bool Continue_Flag1;
        if (Full_Repaint_Flag)
        THEN
            GUI_Presentation_API::Clear(Widget_Ids::SimStats_Draw_Area); 

            Continue_Flag1 = SimStats_Ptr->Iter_To_First_Ind_Series();
            Draw_Series_X_Axis(Last_Series_Ptr, Colour_Ids::Black);
            
//          Draw the Y Axis. The axis ticks and increments are the same as for the X Axis
//          so we call Draw_T_Axis with the required arguments, rather than just calling 
//          Draw_Series_Y_Axis

            int Axis_Number = 1;
            Draw_Y_Axis(First_Tick, 
                        Last_Tick, 
                        Increment, 
                        Colour_Ids::Black, 
                        string("Base Time"),  // Axis Name
                        Axis_Number);

//          Display the Sensitivity
 
//            TBitmap_SimStats_Ptr->Canvas()->TextOut(Sensitivity_Text_X, Sensitivity_Text_Y, 
//            string(string("Sensitivity = ") + To_String(Manhattan_Plot_Sensitivity)));   
            
//          Draw the diagonal bounding the Manhattan Distance Plot
                      
            X_Y_Coords  Start, End;
            Start.X = First_Tick;
            Start.Y = First_Tick;
            End.X = Last_Tick;
            End.Y = Last_Tick;
            Screen_Loc Screen_Loc_Start = Map_X_Y_Coords_To_Screen_Loc(Start, End, Start); 
            Screen_Loc Screen_Loc_End = Map_X_Y_Coords_To_Screen_Loc(Start, End, End);     
            Join_Screen_Locs(Screen_Loc_Start, Screen_Loc_End, 1, Colour_Ids::Blue);
            
        ELSE
            Continue_Flag1 = SimStats_Ptr->Iter_To_Last_Ind_Series();  // Only plot data for last Series
        ENDIF
        
//      Now plot the required data for either: a) All series, or b) the last Series,
//      depending on where the Series Iterator of the Simulation Statistics oblect has been left.
        
        X_Y_Coords   Low_Vals, High_Vals;   
        Low_Vals.X = First_Tick;
        Low_Vals.Y = First_Tick;
        High_Vals.X = Last_Tick;
        High_Vals.Y = Last_Tick;

        int Loop_Count = 0;            
        for  (;
            Continue_Flag1 == true;
            Continue_Flag1 = SimStats_Ptr->Iter_To_Next_Ind_Series())
        BEGIN
            double I_Dummy;
            double D_Dummy;
            double X_Index;   //  Index for the current Value in the current Indexed Series
            double Y_Index;   //  Index for the last Value in the current Indexed Series
            double  Value;
            Indexed_Series* Series_Ptr = SimStats_Ptr->Get_Current_Indexed_Series_Ptr();
            Series_Ptr->Get_Series_Ranges(I_Dummy, Y_Index,D_Dummy, D_Dummy);

//          Plot each of the points for the current Series     
         
            for (bool Continue_Flag2 = Series_Ptr->Get_First_Data_Pair(X_Index, Value);
                 Continue_Flag2 == true;
                 Continue_Flag2 = Series_Ptr->Get_Next_Data_Pair(X_Index, Value))
            BEGIN        

//              Transform co-ordinates for horizontal display so that 
//              X = Current Time
//              Y = Age of the point we are plotting

//              The transformation is:-
//              Y  ->  Y - X
//              X  ->  Y

                double Translated_Y = Y_Index - X_Index;
                double Translated_X  = Y_Index;

                X_Y_Coords  Coords;
                Coords.X = Translated_X;
                Coords.Y = Translated_Y;

                Screen_Loc Screen_Pos = Map_X_Y_Coords_To_Screen_Loc(Low_Vals, High_Vals, Coords);
                   
//              Normalise based on the current maximum possible Manhattan Distance (2 * num Mols).
                
                double Normalised_Manhattan_Distance = Value / (2 * Current_Num_Mols); 

//                int Brightness = static_cast<int> 
//                    (0xFFFF * (1 - min (double(1), Manhattan_Plot_Sensitivity * Normalised_Manhattan_Distance)));
                int Brightness = static_cast<int> 
                    (0xFFFF * min (double(1), Manhattan_Plot_Sensitivity * Normalised_Manhattan_Distance));

      
                GUI_Presentation_API::Set_Foreground_Colour(Widget_Ids::SimStats_Draw_Area,
                                                            Brightness, Brightness, Brightness);
                GUI_Presentation_API::Draw_Rectangle(Widget_Ids::SimStats_Draw_Area,
                                                    true, Screen_Pos.X, Screen_Pos.Y, 4, 4);
          
//                Draw_Rectangle(Screen_Pos, 2, 2, Colour);                                   
            END
            
//          Display the plot periodically as we go along so that the user can see the progress
            
            if (Loop_Count++%100 == 0)
            THEN

//              Text to display during plotting / replotting (but not while the simulation is running)
                if (not Simulation_Running)
                THEN
//                    TBitmap_SimStats_Ptr->Canvas()->Font()->Set_Color(clRed());  
//                    TBitmap_SimStats_Ptr->Canvas()->TextOut(Plotting_Text_X, Plotting_Text_Y,string("PLOTTING - PLEASE WAIT..."));
                ENDIF
                GUI_Presentation_API::Draw_Pixmap(Widget_Ids::SimStats_Draw_Area);
            ENDIF
        END      

//      Remove Plotting in progress text

        Screen_Loc Loc;
        Loc.X = Plotting_Text_X + 50;
        Loc.Y = Plotting_Text_Y + 10;
//        Draw_Rectangle(Loc, 20, 250, clWhite());   
    ENDIF
}
//-----------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::Display_Statistics(Display_Option Option, bool Simulation_Running)

//  Display the Statistics for the currently selected group

{
    TRACE("Form_Simulation_Statistics::Display_Statistics");

    Last_Display_Time = Krnl_Vis::Reactor_::Get_Time();
    if (Current_Display_Group == Manhattan_Plots)
    THEN
        Display_Manhattan_Plot(Option, Simulation_Running);
    ELSEIF (Current_Display_Group == Series_Plots)
    THEN
        Display_Series_Plots(Option); 
    ENDIF

    GUI_Presentation_API::Draw_Pixmap(Widget_Ids::SimStats_Draw_Area);
//    List_Groups();
}
//-----------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::Draw_X_Axis(double First_Tick, double Last_Tick, double Increment, 
                                                Colour_Id Display_Colour, string Axis_Name)

//  Draw an X axis with a specified first / last tick, increment and colour

{

//  Draw the main line for the X Axis

    Screen_Loc Start;
    Screen_Loc End;
    Start.X = SimStats_Plot_Area.Left;
    Start.Y = SimStats_Plot_Area.Bottom;
    End.X =   SimStats_Plot_Area.Right;
    End.Y =   SimStats_Plot_Area.Bottom;
   
    Join_Screen_Locs(Start, End, 3, Display_Colour);  
 
//  Draw the ticks and labels

    double Current_Tick = First_Tick;
    
    while (Current_Tick <= Last_Tick)
    BEGIN  
        Screen_Loc Start_Tick;
        Screen_Loc End_Tick;
    
        Start_Tick.X =  Map_X_Val_To_Screen_X(First_Tick, Last_Tick, Current_Tick);
        Start_Tick.Y =  SimStats_Plot_Area.Bottom;
    
        End_Tick.X =  Start_Tick.X;
        End_Tick.Y =  Start_Tick.Y + 20;

        Join_Screen_Locs(Start_Tick, End_Tick, 3, Display_Colour); 
         
        GUI_Presentation_API::Text_Out(Widget_Ids::SimStats_Draw_Area, 
                                       End_Tick.X-10, 
                                       End_Tick.Y+5, 
                                       To_String(Current_Tick)); 
        
        Current_Tick = Current_Tick + Increment;
    END
    
//  Output the label for the axis

    int Label_X = ((SimStats_Plot_Area.Right + SimStats_Plot_Area.Left)/2) - 50;
    int Label_Y = SimStats_Plot_Area.Bottom + 37;
    
    GUI_Presentation_API::Text_Out(Widget_Ids::SimStats_Draw_Area, Label_X, Label_Y, Axis_Name);
}
//------------------------------------------------------------------------------------------------------------------

void  Form_Simulation_Statistics::Draw_Y_Axis(double First_Tick, 
                                              double Last_Tick, 
                                              double Increment, 
                                              Colour_Id Display_Colour, 
                                              string Axis_Name, 
                                              int Axis_Number)
                                              
//  Draw Y axis with the specified properties

{
    int  Tick_Length;
    int Text_Offset;

//  Draw the main line for the Y Axis. 

    Screen_Loc Start_Axis;
    Screen_Loc End_Axis;
    
    if (Axis_Number == 1)   
    THEN
        Start_Axis.X = SimStats_Plot_Area.Left;
        End_Axis.X =   SimStats_Plot_Area.Left;
        Tick_Length = -20;
        Text_Offset = -30;
    ELSE 
        Start_Axis.X = SimStats_Plot_Area.Right + (Axis_Number - 2)*45;
        End_Axis.X =   SimStats_Plot_Area.Right + (Axis_Number - 2)*45;
        Tick_Length = 20;   
        Text_Offset = -15;
    ENDIF   
    
    Start_Axis.Y = SimStats_Plot_Area.Bottom;
    End_Axis.Y =   SimStats_Plot_Area.Top;
    
    Join_Screen_Locs(Start_Axis, End_Axis, 3, Display_Colour);  

//  Draw the ticks and labels
   
    double Current_Tick = First_Tick;
    while (Current_Tick <= Last_Tick)
    BEGIN  
        Screen_Loc Start_Tick;
        Screen_Loc End_Tick;
    
        Start_Tick.X =   Start_Axis.X;
        Start_Tick.Y =  Map_Y_Val_To_Screen_Y(First_Tick, Last_Tick, Current_Tick);
   
        End_Tick.X =  Start_Tick.X + Tick_Length;     
        End_Tick.Y =  Start_Tick.Y;

        Join_Screen_Locs(Start_Tick, End_Tick, 3, Display_Colour); 
         
        GUI_Presentation_API::Text_Out(Widget_Ids::SimStats_Draw_Area, 
                                       End_Tick.X + Text_Offset,
                                       End_Tick.Y + 5,
                                       To_String(Current_Tick));

        Current_Tick = Current_Tick + Increment;
    END
}                                              
//---------------------------------------------------------------------------------------------------------------- 

void    Form_Simulation_Statistics::Draw_Series_X_Axis(Indexed_Series* Series_Ptr, Colour_Id Display_Colour)

//  Draw the X axis for the specified Series on the working Pixmap

{
    double First_Tick, Last_Tick, Increment; 
    Series_Ptr->Get_Index_Axis(First_Tick, Last_Tick, Increment);
    
    Draw_X_Axis(First_Tick, Last_Tick, Increment, Display_Colour, 
                To_String(Series_Ptr->Get_Index_Name()));
} 
//---------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::Draw_Series_Y_Axis(Indexed_Series* Series_Ptr, 
                                                       Colour_Id Display_Colour, 
                                                       string Axis_Name, 
                                                       int Axis_Number)

//  Draw the Y axis for the specified Series

{
    double First_Tick, Last_Tick, Increment; 
    Series_Ptr->Get_Value_Axis(First_Tick, Last_Tick, Increment);
      
    Draw_Y_Axis(First_Tick, 
                Last_Tick, 
                Increment, 
                Display_Colour, 
                Axis_Name, 
                Axis_Number);
 } 
//---------------------------------------------------------------------------------------------------------------
Form_Simulation_Statistics::Form_Simulation_Statistics()

//  Constructor

{
    TRACE("Form_Simulation_Statistics::Form_Simulation_Statistics");
//  Get the pointer to the PaintBox and the Simulation Statistics Object      
   
    SimStats_Ptr = Simulation_Statistics::Get_Ptr();    
    
//  Ensure Bottom is lower than top

    SimStats_Plot_Area.Top = 0;  //  INITIALISATION BUG TEST
    if (SimStats_Plot_Area.Bottom <= SimStats_Plot_Area.Top) 
    THEN
        SimStats_Plot_Area.Bottom = SimStats_Plot_Area.Top +10;
    ENDIF
    
//  Ensure Left is to the left of Right
        
    SimStats_Plot_Area.Left = 0;  //  INITIALISATION BUG TEST
    if (SimStats_Plot_Area.Right <= SimStats_Plot_Area.Left) 
    THEN
        SimStats_Plot_Area.Right = SimStats_Plot_Area.Left +10;
    ENDIF
  
//  Setup the Display Groups

    Display_Group_Spec* Disp_Group_Ptr;
    CREATE_OBJECT(Display_Group_Spec, Disp_Group_Ptr = new Display_Group_Spec);
    Disp_Group_Ptr->Group_Id = Series_Plots;
    Disp_Group_Ptr->Group_Name = string("Series Plots");
    List_Of_Display_Group_Specs.push_back(Disp_Group_Ptr);
       
    CREATE_OBJECT(Display_Group_Spec, Disp_Group_Ptr = new Display_Group_Spec);
    Disp_Group_Ptr->Group_Id = Manhattan_Plots;
    Disp_Group_Ptr->Group_Name = string("Manhattan Plot");
    List_Of_Display_Group_Specs.push_back(Disp_Group_Ptr);
   
    Current_Display_Group = Series_Plots;

    Curr_SerList_Name_Tx_ = SimStats_Ptr->Get_Default_SerList_Nm();

    Manhattan_Plot_Sensitivity = 10;

    SimStats_Has_Been_Displayed = false;
  
    Init_Series_Display();

//  Initialise the Series to be dispalyed

    for (int Continue_Flag1 = SimStats_Ptr->Iter_To_First_Ind_Series_List();
         Continue_Flag1 == true;
         Continue_Flag1 = SimStats_Ptr->Iter_To_Next_Ind_Series_List())
    BEGIN
        for (int Continue_Flag2 = SimStats_Ptr->Iter_To_First_Ind_Series();
            Continue_Flag2 == true;
            Continue_Flag2 = SimStats_Ptr->Iter_To_Next_Ind_Series())
        BEGIN
            Indexed_Series*  Current_Series_Ptr = SimStats_Ptr->Get_Current_Indexed_Series_Ptr();
            if ((Current_Series_Ptr->Get_Indexed_Series_Name() == string("Molecules"))
                        or
            (Current_Series_Ptr->Get_Indexed_Series_Name() == string("Molecule Types")) 
                        or
            (Current_Series_Ptr->Get_Indexed_Series_Name() == string("Interaction Rate")) 
                        or
            (Current_Series_Ptr->Get_Indexed_Series_Name() == string("Realisable Int. Types")) 
                    or
            (Current_Series_Ptr->Get_Indexed_Series_Name() == string("Manhattan Dist. (Composome)")))
            THEN
                Toggle_Series_Display(Current_Series_Ptr);
            ENDIF
        END
    END
}
//---------------------------------------------------------------------------------------------------------------

Form_Simulation_Statistics*  Form_Simulation_Statistics::Get_Ptr()

//  Get the pointer to the singleton Form_Simulation_Statistics object.
//  Its a fatal error if the object has not yet been created.

{
    Assert(Instance_Ptr not_eq 0,"Form_Simulation_Statistics::Get_Ptr: Called before an instance has been created");
    return (Instance_Ptr);
}
//---------------------------------------------------------------------------------------------------------------                

void    Form_Simulation_Statistics::Get_ListView_Groups_Selected_Row(list<string>& List_Of_Col_Texts, 
                                                                     Display_Group_Spec*& Disp_Group_Spec_Ptr)

//   Get the display texts and a pointer to data for the selected row in the Groups ListView

{
    void*  Data_Ptr;
    GUI_Presentation_API::Get_Selected_Row(Widget_Ids::SimStats_Groups_ListView,
                                           List_Of_Col_Texts,
                                           Data_Ptr);
    Disp_Group_Spec_Ptr = static_cast <Display_Group_Spec*> (Data_Ptr);
}
//----------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::Get_ListView_Series_Selected_Row(list<string>& List_Of_Col_Texts, 
                                                                     Indexed_Series*& Indexed_Series_Ptr)

//   Get the display texts and a pointer to data for the selected row in the Series ListView

{
    void*  Data_Ptr;
    GUI_Presentation_API::Get_Selected_Row(Widget_Ids::SimStats_Series_ListView,
                                           List_Of_Col_Texts,
                                           Data_Ptr);
    Indexed_Series_Ptr = static_cast <Indexed_Series*> (Data_Ptr);
}
//----------------------------------------------------------------------------------------------------------------

double Form_Simulation_Statistics::Get_Manhattan_Plot_Sensitivity()

{
    return Manhattan_Plot_Sensitivity;
}
//-----------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::Init_Series_Display()

//  Initialise the series display to be consistent with the 
//  way the Indexed Series are setup

{
    TRACE("Form_Simulation_Statistics::Initialise_Series_Display");

    Available_Plot_Colours.clear();
    Available_Plot_Colours.push_back(Colour_Ids::Black);
    Available_Plot_Colours.push_back(Colour_Ids::Blue);
    Available_Plot_Colours.push_back(Colour_Ids::Green);
    Available_Plot_Colours.push_back(Colour_Ids::Maroon);
    Available_Plot_Colours.push_back(Colour_Ids::Red);  

    List_Of_Series_To_Plot.clear();

    for (int Cont1_Fl = SimStats_Ptr->Iter_To_First_Ind_Series_List();
         Cont1_Fl == true;
         Cont1_Fl = SimStats_Ptr->Iter_To_Next_Ind_Series_List())
    BEGIN
        for (int Cont2_Fl = SimStats_Ptr->Iter_To_First_Ind_Series();
            Cont2_Fl == true;
            Cont2_Fl = SimStats_Ptr->Iter_To_Next_Ind_Series())
        BEGIN
            Indexed_Series*  Series_Pt = SimStats_Ptr->Get_Current_Indexed_Series_Ptr();
            if (Series_Pt->Is_For_Display())
            THEN                   
                Assert (Available_Plot_Colours.empty() not_eq true,
                    "Form_Simulation_Statistics::Initialise_Series_Display - Attempt to plot more series than there are plot colours");           
                Series_And_Colour_To_Plot  Plot_Req;
                Plot_Req.Series_Ptr = Series_Pt;
                Plot_Req.Plot_Colour = Available_Plot_Colours.front();
                Available_Plot_Colours.pop_front();
                List_Of_Series_To_Plot.push_back(Plot_Req);
            ENDIF
        END
    END
}
//---------------------------------------------------------------------------------------------------------------

bool   Form_Simulation_Statistics::Is_On_Active_Page()

//  Return true if the Simulation Statistics Tab is on the currently active page

{   

    string Name = GUI_Presentation_API::Get_Active_Page_Label();
    if (Name == string("Simulation Statistics"))
    THEN
        return true;
    ELSE
        return false;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::List_Series()

//  List the Series in the ListView and link each to its Data_Series

{
    TRACE("Form_Simulation_Statistics::List_Series()");
    GUI_Presentation_API::Clear(Widget_Ids::SimStats_Series_ListStore);

    for (bool Cont_Ser_Fl = SimStats_Ptr->Iter_To_First_Ind_Series();
        Cont_Ser_Fl == true;
        Cont_Ser_Fl = SimStats_Ptr->Iter_To_Next_Ind_Series())
    BEGIN
        Indexed_Series*  Curr_Ser_Pt = SimStats_Ptr->Get_Current_Indexed_Series_Ptr();
        TRACEVAL(Curr_Ser_Pt);
        Display_Series_Name(Curr_Ser_Pt );
    END
}    
//-----------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::ListView_Group_DblClick()

//  Handler for Double Click on the Groups Listview

{
    TRACE("Form_Simulation_Statistics::ListView_Group_DblClick");
    list<string> List_Of_Col_Texts_Curr;
    list<string> List_Of_Col_Texts_New;
    Display_Group_Spec* Disp_Group_Spec_Ptr;
    Get_ListView_Groups_Selected_Row(List_Of_Col_Texts_Curr, Disp_Group_Spec_Ptr);

    Current_Display_Group = Disp_Group_Spec_Ptr->Group_Id;
    List_Groups();

    if (Current_Display_Group == Manhattan_Plots)
    THEN
        GUI_Presentation_API::Clear(Widget_Ids::SimStats_Series_ListStore);
        GUI_Presentation_API::Clear(Widget_Ids::SimStats_SerList_ListStore);
        Display_Statistics(Full, false);
    ELSEIF (Current_Display_Group == Series_Plots)
    THEN
        SimStats_Ptr->Set_Current_Ind_Series_List(Curr_SerList_Name_Tx_);
        List_Series_Lists();
        List_Series();
        Display_Statistics(Full, false);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::ListView_SerList_Select()

//  Handler for Selecting  the Series List Listview

{
    TRACE("Form_Simulation_Statistics::ListView_SerList_Select");
    list<string> String_Ls;
    if (GUI_Presentation_API::Get_Selected_Row(Widget_Ids::SimStats_SerList_ListView, 
                                      String_Ls))
    THEN 
        Curr_SerList_Name_Tx_ = String_Ls.front();
        SimStats_Ptr->Set_Current_Ind_Series_List(Curr_SerList_Name_Tx_);
        List_Series();
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::ListView_Series_DblClick()

//  Handler for Double Click event on Series ListView 
//  Toggle display of the selected series.

{
    TRACE("Form_Simulation_Statistics::ListView_Series_DblClick")
    list<string> List_Of_Col_Texts_Curr;
    list<string> List_Of_Col_Texts_New;
    Indexed_Series* Indexed_Series_Ptr;

    Get_ListView_Series_Selected_Row(List_Of_Col_Texts_Curr, Indexed_Series_Ptr);
    TRACEVAL(Indexed_Series_Ptr);

    if ((Available_Plot_Colours.size() < 1) and Indexed_Series_Ptr->Is_For_Display() == false) 
    THEN        
        return;
    ELSE       
        string Plot_Text = Toggle_Series_Display(Indexed_Series_Ptr);
        TRACEVAL(Plot_Text);
        List_Of_Col_Texts_New.push_back(List_Of_Col_Texts_Curr.front());
        List_Of_Col_Texts_New.push_back(Plot_Text);
        GUI_Presentation_API::Update_Selected_Row(Widget_Ids::SimStats_Series_ListView, 
                                                  List_Of_Col_Texts_New,
                                                  Indexed_Series_Ptr);
        Display_Series_Plots(Full);
    ENDIF

    GUI_Presentation_API::Draw_Pixmap(Widget_Ids::SimStats_Draw_Area);
}
//----------------------------------------------------------------------------------------------------------------
     
void        Form_Simulation_Statistics::List_Groups()

//  List the Statistics Groups

{    
    TRACE("Form_Simulation_Statistics::List_Groups");
    GUI_Presentation_API::Clear(Widget_Ids::SimStats_Groups_ListStore);

    vector<Display_Group_Spec*>::iterator  Iter;
    
    for (Iter = List_Of_Display_Group_Specs.begin();
         Iter not_eq List_Of_Display_Group_Specs.end();
         Iter++)
    BEGIN
        list<string>   List_Of_Strings;
        List_Of_Strings.push_back((*Iter)->Group_Name);

//      Indicate whether the Group is currently displayed

        if (Current_Display_Group == (*Iter)->Group_Id)
        THEN
            List_Of_Strings.push_back(string("Yes"));
        ELSE
            List_Of_Strings.push_back(string("No"));
        ENDIF

        GUI_Presentation_API::Add_Row(Widget_Ids::SimStats_Groups_ListStore, 
                                      List_Of_Strings,
                                      static_cast <void*> (*Iter)); 

//      Make sure that the Series are listed if the Series Plots group is the current group
                
/*        if((Current_Display_Group == Series_Plots) and ((*Iter)->Group_Id == Series_Plots))
        THEN   
//            List_Series();
//            List_Series_Lists();
        ENDIF*/
    END
}
//----------------------------------------------------------------------------------------------------------------

void  Form_Simulation_Statistics::List_Series_Lists()

//  List the Indexed Series Lists

{
    TRACE("Form_Simulation_Statistics::List_Series_Lists");
    GUI_Presentation_API::Clear(Widget_Ids::SimStats_SerList_ListStore);

    for (bool Cont_Fl = SimStats_Ptr->Iter_To_First_Ind_Series_List();
         Cont_Fl == true;
         Cont_Fl = SimStats_Ptr->Iter_To_Next_Ind_Series_List())
    BEGIN
        list<string>   String_Ls;
        string Name_Tx = SimStats_Ptr->Get_Curr_SerList_Name();
        String_Ls.push_back(Name_Tx);

        if (SimStats_Ptr->Get_Curr_SerList_Usr_Access()) // Only show series list if it is 
                                                         // intended for user visibility
        THEN
            GUI_Presentation_API::Add_Row(Widget_Ids::SimStats_SerList_ListStore, 
                                          String_Ls); 
        ENDIF
    END

    GUI_Presentation_API::Select_Row(Widget_Ids::SimStats_SerList_ListView, Curr_SerList_Name_Tx_ );
}
//---------------------------------------------------------------------------------------------------------------
  
void    Form_Simulation_Statistics::Join_Screen_Locs(const Screen_Loc Start_Screen_Loc,
                                                     const Screen_Loc End_Screen_Loc,
                                                     const int  Line_Width,
                                                     const Colour_Id Line_Colour)
                                                     
//  Draw a line joining the specified Screen Locations

{
//    TPoint   Connection_Elements[2];

//  Start of Line

//    Connection_Elements[0].x = Start_Screen_Loc.X;
//    Connection_Elements[0].y = Start_Screen_Loc.Y;
//    Connection_Elements[1].x = End_Screen_Loc.X;
//    Connection_Elements[1].y = End_Screen_Loc.Y;

    GUI_Presentation_API::Set_Foreground_Colour(Widget_Ids::SimStats_Draw_Area, 
                                                Line_Colour);

    GUI_Presentation_API::Draw_Line(Widget_Ids::SimStats_Draw_Area, 
                                    Start_Screen_Loc.X, 
                                    Start_Screen_Loc.Y,
                                    End_Screen_Loc.X,
                                    End_Screen_Loc.Y);
}
//---------------------------------------------------------------------------------------------------------------

void  Form_Simulation_Statistics::Make()

//  Create the Form_Simulation_Statistics singleton object

{
    TRACE("Form_Simulation_Statistics::Make");
    Assert(Instance_Ptr == 0,"Form_Simulation_Statistics::Make -  Invalid call; Form_Simulation_Statistics has already been created");
    CREATE_OBJECT(Form_Simulation_Statistics, Instance_Ptr = new Form_Simulation_Statistics);   
}
//---------------------------------------------------------------------------------------------------------------

Form_Simulation_Statistics::Screen_Loc   
    Form_Simulation_Statistics::Map_X_Y_Coords_To_Screen_Loc  (const X_Y_Coords& Min_Vals, 
                                                               const X_Y_Coords& Max_Vals, 
                                                               const X_Y_Coords& Pair)
    
//  Determine the Screen Location that corresponds to an X, Y coordinate pair

{
    double X_Axis_Range = Max_Vals.X - Min_Vals.X;
    double Y_Axis_Range = Max_Vals.Y - Min_Vals.Y;
    double X_Axis_Screen_Range = SimStats_Plot_Area.Right - SimStats_Plot_Area.Left;
    double Y_Axis_Screen_Range = SimStats_Plot_Area.Top - SimStats_Plot_Area.Bottom;
     
    Screen_Loc Ret_Screen_Loc;
    
    Ret_Screen_Loc.X =  static_cast<int> 
                        (SimStats_Plot_Area.Left
                      + ((Pair.X - Min_Vals.X) * X_Axis_Screen_Range/X_Axis_Range));
    Ret_Screen_Loc.Y =  static_cast<int> 
                        (SimStats_Plot_Area.Bottom
                      + ((Pair.Y - Min_Vals.Y) * Y_Axis_Screen_Range/Y_Axis_Range));
                     
    return Ret_Screen_Loc;               
} 
//---------------------------------------------------------------------------------------------------------------
int    Form_Simulation_Statistics::Map_X_Val_To_Screen_X(const double& X_Low, 
                                                         const double& X_High, 
                                                         const double& X_Val)

//  Return the screen X co-ordinate that corresponds to the specified X Value, taking account
//  of the specified high and low values for X

{
    int Screen_Range = SimStats_Plot_Area.Right - SimStats_Plot_Area.Left;   
    return static_cast <int> (SimStats_Plot_Area.Left + ((X_Val - X_Low) * Screen_Range)/(X_High - X_Low));
}
//---------------------------------------------------------------------------------------------------------------
int    Form_Simulation_Statistics::Map_Y_Val_To_Screen_Y(const double& Y_Low, 
                                                         const double& Y_High, 
                                                         const double& Y_Val)

//  Return the screen Y co-ordinate that corresponds to the specified Y Value, taking account
//  of the specified high and low values for Y

{
    int Screen_Range = SimStats_Plot_Area.Bottom - SimStats_Plot_Area.Top;  
    return static_cast <int> (SimStats_Plot_Area.Bottom - ((Y_Val - Y_Low) * Screen_Range)/(Y_High - Y_Low));
}
//---------------------------------------------------------------------------------------------------------------

unsigned int    Form_Simulation_Statistics::Num_Series_For_Display()

//  Return the number of Series that are currently marked for display
//  Iterates over Series, so DO NOT USE WITHIN AN EXISTING SERIES ITERATION

{   
    int Series_Counter = 0;     
    
    for (int Continue_Flag1 = SimStats_Ptr->Iter_To_First_Ind_Series_List();
    Continue_Flag1 == true;
    Continue_Flag1 = SimStats_Ptr->Iter_To_Next_Ind_Series_List())
    BEGIN  
        for (int Continue_Flag2 = SimStats_Ptr->Iter_To_First_Ind_Series();
            Continue_Flag2 == true;  
            Continue_Flag2 = SimStats_Ptr->Iter_To_Next_Ind_Series())
        BEGIN
            if (SimStats_Ptr->Get_Current_Indexed_Series_Ptr()->Is_For_Display() == true)
            THEN  
                Series_Counter++;
            ENDIF
        END
    END
    return Series_Counter;
}
//---------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::Page_Select()

//  For the first time only, display the Simulation Statistics when its notebook page is selected

{
    if (!SimStats_Has_Been_Displayed)
    THEN
        List_Groups();
        Curr_SerList_Name_Tx_ = SimStats_Ptr->Get_Default_SerList_Nm();
        List_Series_Lists();
//      Selected_SerList_ = SimStats_Ptr->Get_Default_SerList_Nm();
    
        SimStats_Ptr->Set_Current_Ind_Series_List(Curr_SerList_Name_Tx_);

//        GUI_Presentation_API::Select_Row(Widget_Ids::SimStats_SerList_ListView, Selected_SerList_ );
//        list<string>  Texts;
//        GUI_Presentation_API::Get_Row_N(Widget_Ids::SimStats_SerList_ListStore, 1 , Texts);
        List_Series();
        Display_Statistics(Form_Simulation_Statistics::Full,
                                            false);  // Assume simulation not running
        SimStats_Has_Been_Displayed = true;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Statistics::Plot_Indexed_Series(Indexed_Series* Series_Ptr, 
                                                        Colour_Id Display_Colour, 
                                                        bool Replot_Flag,
                                                        int Max_Index)
//                                                      Max_Index is the highest possible index.
//                                                      If the Index for the last data point is less
//                                                      than Max_Index, then we have missing entries 
//                                                      in the series

//  Plot the Simulation Statistics' Specified Series

{
    TRACE("Form_Simulation_Statistics::Plot_Indexed_Series");

    X_Y_Coords Low_Vals, High_Vals;
    double X_Increment;
    double Y_Increment;
    int Stat_Coll_Intrvl = Krnl_Vis::Scenario_::Get_Statistics_Collection_Interval();
   
    Series_Ptr->Get_Index_Axis(Low_Vals.X, High_Vals.X, X_Increment);
    Series_Ptr->Get_Value_Axis(Low_Vals.Y, High_Vals.Y, Y_Increment);
                      
    X_Y_Coords Data_Pair1, Data_Pair2;

//  Plotting starts at the first data (Index + Value) pair

    Series_Ptr->Get_First_Data_Pair(Data_Pair1.X, Data_Pair1.Y);             
    Screen_Loc Screen_Loc1 = Map_X_Y_Coords_To_Screen_Loc(Low_Vals, High_Vals, Data_Pair1);
    Screen_Loc Screen_Loc2;
                 
    while (Series_Ptr->Get_Next_Data_Pair(Data_Pair2.X, Data_Pair2.Y))
    BEGIN       
        Screen_Loc2 = Map_X_Y_Coords_To_Screen_Loc(Low_Vals, High_Vals, Data_Pair2); 

//      First check whether there are 'missing' entries
        if ((Data_Pair2.X - Data_Pair1.X) > Stat_Coll_Intrvl)
        THEN
//          If missing entry does not mean zero, then we don't know what it means!
            Assert(Series_Ptr->No_Entry_Means_Zero(),
                    "Form_Simulation_Statistics::Plot_Indexed_Series - missing entry in series, but not defined to mean zero");

//          We must construct the plot for the missing entries.
//          First, between the last data point and the 'missing' one after it
            X_Y_Coords  Missing1;
            Missing1.X = Data_Pair1.X + Stat_Coll_Intrvl;
            Missing1.Y = 0;
            Screen_Loc ScrnLoc_Miss1 = Map_X_Y_Coords_To_Screen_Loc(Low_Vals, High_Vals, Missing1); 
            Join_Screen_Locs(Screen_Loc1, ScrnLoc_Miss1,1, Display_Colour);

//          Now join the two missing data points
            X_Y_Coords  Missing2;
            Missing2.X = Data_Pair2.X - Stat_Coll_Intrvl;
            Missing2.Y = 0;
            Screen_Loc ScrnLoc_Miss2 = Map_X_Y_Coords_To_Screen_Loc(Low_Vals, High_Vals, Missing2); 
            Join_Screen_Locs(ScrnLoc_Miss1, ScrnLoc_Miss2,1, Display_Colour);

//          Now join the second missing data point to the second actual point
            Join_Screen_Locs(ScrnLoc_Miss2, Screen_Loc2, 1, Display_Colour);
        ELSE  
//          Normal case, no missing entries
            Assert((Data_Pair2.X - Data_Pair1.X) == Stat_Coll_Intrvl,
                    "Form_Simulation_Statistics::Plot_Indexed_Series - Unexpected entry in series");

            Join_Screen_Locs(Screen_Loc1, Screen_Loc2, 1, Display_Colour);
        ENDIF
        Screen_Loc1 = Screen_Loc2;
        Data_Pair1 = Data_Pair2;
    END

    Assert (Data_Pair2.X <= Max_Index, 
                "Form_Simulation_Statistics::Plot_Indexed_Series - Data point index out of expected range");

//  If we have got to the last data point, it is possible 
//  that there are missing entries between it and the Max_Index (end of range)
//  for which we are to plot data. The missing entries should be treated as zero

    if (Max_Index > (Data_Pair2.X + Stat_Coll_Intrvl))
    THEN
        Assert(Series_Ptr->No_Entry_Means_Zero(),
                    "Form_Simulation_Statistics::Plot_Indexed_Series - missing entry at end of series, but not defined to mean zero");
  
//      We must construct the plot for the missing entries.
//      First, between the last data point and the 'missing' one after it
        X_Y_Coords  MissingA;
        MissingA.X = Data_Pair2.X + Stat_Coll_Intrvl;
        MissingA.Y = 0;
        Screen_Loc ScrnLoc_MissA = Map_X_Y_Coords_To_Screen_Loc(Low_Vals, High_Vals, MissingA); 
        Join_Screen_Locs(Screen_Loc2, ScrnLoc_MissA,1, Display_Colour);

//      Now join the first missing data point to the notional one at Max_Index
        X_Y_Coords  MissingB;
        MissingB.X = Max_Index;
        MissingB.Y = 0;
        Screen_Loc ScrnLoc_MissB = Map_X_Y_Coords_To_Screen_Loc(Low_Vals, High_Vals, MissingB); 
        Join_Screen_Locs(ScrnLoc_MissA, ScrnLoc_MissB,1, Display_Colour);
    ENDIF
}
//---------------------------------------------------------------------------------------------------------------

void Form_Simulation_Statistics::Save_Display_To_File(string* Filename_Ptr)

{
    string Filename = *Filename_Ptr + string(".ser");

    Display_Manhattan_Plot(Full, false);
    Filename = *Filename_Ptr + string(".mnh");
}
//----------------------------------------------------------------------------------------------------------------

void Form_Simulation_Statistics::Set_Manhattan_Plot_Sensitivity(double Sensitivity)

{
    Manhattan_Plot_Sensitivity = Sensitivity;
}
//-------------------------------------------------------------------------------------------------------------------
string   Form_Simulation_Statistics::Toggle_Series_Display(Indexed_Series* Data_Series_Ptr )

//  Display / Undisplay the specified series

//  Toggle the 'Is For Display' indicator for the selected Series, and update the 
//  Series ListView accordingly. 

{
    TRACE("Form_Simulation_Statistics::Toggle_Series_Display");

    Data_Series_Ptr->Toggle_Series_Display();
    
    string Plot_Text;
    if (Data_Series_Ptr->Is_For_Display())
    THEN
        TRACEVAL("Is For Display");
        Plot_Text = "Yes";                    
        Assert (Available_Plot_Colours.empty() not_eq true,
                "Form_Simulation_Statistics::Toggle_Series_Display - Attempt to plot more series than there are plot colours");           
        Series_And_Colour_To_Plot  Plot_Requirement;
        Plot_Requirement.Series_Ptr = Data_Series_Ptr;
        Plot_Requirement.Plot_Colour = Available_Plot_Colours.front();
        Available_Plot_Colours.pop_front();
        List_Of_Series_To_Plot.push_back(Plot_Requirement);
    ELSE
        TRACEVAL("Is Not For Display");
        Plot_Text = "No";            
        list <Series_And_Colour_To_Plot>::iterator  Iter;
        TRACEVAL(List_Of_Series_To_Plot.size());
        for (Iter = List_Of_Series_To_Plot.begin();
                Iter not_eq List_Of_Series_To_Plot.end();
                Iter++)
        BEGIN
            if ((*Iter).Series_Ptr == Data_Series_Ptr)
            THEN
                TRACEVAL(Data_Series_Ptr);
                break;
            ENDIF
        END
        TRACEVAL((*Iter).Plot_Colour);
        Available_Plot_Colours.push_back((*Iter).Plot_Colour);
        TRACEVAL(string("After Push Back"));
        List_Of_Series_To_Plot.erase(Iter);
        TRACEVAL(string("After erase"));
    ENDIF
   
    return Plot_Text;
}

