/****************************************************************************
*
*     Module Filename:  GUI_Presentation.h
*     Module Name:      GUI_Presentation
*     Description:      Implementation of the SimSoup Presentation Layer
*
*     Author:           Chris Gordon-Smith
*
*     Copyright (C) 2007, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*      Amendment History:- 
*
*              Version    Date                        Update Description
*              =======         std::string filename = dialog.get_filename(); ===========                 ==================
*
*
*
*
****************************************************************************/
#ifndef GUI_PRESENTATION_H
#define GUI_PRESENTATION_H

class  Drawing_Area; 
class  List_Store;
class  Indexed_Series;

class   GUI_Presentation 
{
    friend class GUI_Presentation_API;  

//===== Relationships with other classes ===================================================================

    private:

//      Main Window
   
        Gtk::Window*     SimSoup_Main_Window_Ptr;
        Gtk::VBox*       SimSoup_Main_VBox_Ptr;
        Gtk::Notebook*   SimSoup_Main_Notebook_Ptr;

        Glib::RefPtr<Gtk::UIManager> SimSoup_Main_UIManager_RefPtr;
        Glib::RefPtr<Gtk::ActionGroup> SimSoup_Main_ActionGroup_RefPtr;

//          Chemistry and Reactor Views Tab

            Gtk::HPaned*    Chem_Reactor_Paned_Ptr;

            Gtk::VPaned*    Chem_View_Paned_Ptr;
            Gtk::HPaned*    Chem_ListViews_Paned_Ptr;

            Drawing_Area*   Chem_Drawing_Area_Ptr;
            Gtk::TreeView*  MolType_ListView_Ptr;
            Gtk::TreeView*  IntType_ListView_Ptr;
            Gtk::ScrolledWindow* MolType_ListView_ScrolledWindow_Ptr;
            Glib::RefPtr<Gtk::ListStore> MolType_ListStore_RefPtr; 
            Gtk::ScrolledWindow* IntType_ListView_ScrolledWindow_Ptr;
            Glib::RefPtr<Gtk::ListStore> IntType_ListStore_RefPtr; 

            Gtk::VPaned*    Reactor_View_Paned_Ptr;
            Gtk::HPaned*    Reactor_ListViews_Paned_Ptr;

            Drawing_Area*   Reactor_Drawing_Area_Ptr;
            Gtk::TreeView*  R_MolType_ListView_Ptr;
            Gtk::TreeView*  R_IntType_ListView_Ptr;
            Gtk::ScrolledWindow* R_MolType_ListView_ScrolledWindow_Ptr;
            Glib::RefPtr<Gtk::ListStore> R_MolType_ListStore_RefPtr; 
            Gtk::ScrolledWindow* R_IntType_ListView_ScrolledWindow_Ptr;
            Glib::RefPtr<Gtk::ListStore> R_IntType_ListStore_RefPtr; 

//          Simulation Statistics Tab

            Gtk::HPaned*    SimStats_Main_Panes_Ptr;
            Gtk::VPaned*    SimStats_ListV_Top_Panes_Pt;
            Gtk::VPaned*    SimStats_ListV_Bot_Panes_Pt;
            Gtk::ScrolledWindow* SimStats_ListView_Groups_ScrolledWindow_Ptr;
            Gtk::ScrolledWindow* SimStats_ListView_SerList_ScrolledWindow_Pt;
            Gtk::ScrolledWindow* SimStats_ListView_Series_ScrolledWindow_Ptr;
            Gtk::TreeView*  SimStats_ListView_Groups_Ptr;
            Gtk::TreeView*  SimStats_ListView_SerList_Pt;
            Gtk::TreeView*  SimStats_ListView_Series_Ptr;
            Glib::RefPtr<Gtk::ListStore> SimStats_ListStore_Groups_RefPtr;  // Reference Counting Ptr to 
                                                                            // data for the Groups ListView
            Glib::RefPtr<Gtk::ListStore> SimStats_ListStore_SerList_RefPtr;  
            Glib::RefPtr<Gtk::ListStore> SimStats_ListStore_Series_RefPtr;  // Reference Counting Ptr to 
                                                                            // data for the Seriess ListView       
            Drawing_Area*   SimStats_Drawing_Area_Ptr;

//          Simulation Control Tab

            Gtk::HPaned*    SimControl_Paned_Ptr;
            Gtk::ScrolledWindow*    SimControl_Actions_ScrolledWindow_Ptr;
            Gtk::TreeView*  SimControl_Actions_ListView_Ptr;

            Gtk::TextView*  SimControl_Log_TextView_Ptr;
            Glib::RefPtr<Gtk::TextBuffer> SimControl_Log_TextBuf_RefPtr;
            Gtk::ScrolledWindow*    SimControl_Log_ScrolledWindow_Ptr;
            Glib::RefPtr<Gtk::ListStore> Simulation_Actions_ListStore_RefPtr;

//          Help Tab

            Gtk::Notebook*   Help_Notebook_Ptr;

            Gtk::TextView*  Help_About_TextView_Ptr;
            Glib::RefPtr<Gtk::TextBuffer> Help_About_TextBuf_RefPtr;
            Gtk::ScrolledWindow*    Help_About_ScrolledWindow_Ptr;

            Gtk::TextView*  Help_Quick_Start_TextView_Ptr;
            Glib::RefPtr<Gtk::TextBuffer> Help_Quick_Start_TextBuf_RefPtr;
            Gtk::ScrolledWindow*    Help_Quick_Start_ScrolledWindow_Ptr;

            Gtk::TextView*  Help_Licence_TextView_Ptr;
            Glib::RefPtr<Gtk::TextBuffer> Help_Licence_TextBuf_RefPtr;
            Gtk::ScrolledWindow*    Help_Licence_ScrolledWindow_Ptr;

//          System Monitor Tab

            Gtk::ScrolledWindow*            SysMon_ScrolledWindow_Ptr;
            Gtk::TreeView*                  SysMon_ListView_Ptr;
            Glib::RefPtr<Gtk::ListStore>    SysMon_ListStore_RefPtr;

//      Data structures for binding the API to the Gtk widgets

        vector <Glib::RefPtr<Gtk::ListStore> >    List_Of_ListStores;
        vector <Gtk::TreeView*> List_Of_ListViews;
        vector <Glib::RefPtr<Gtk::TextBuffer> > List_Of_TextBuffers;
        vector <Drawing_Area*>  List_Of_Drawing_Areas;

        vector <Gdk::Color>     List_Of_Colours;

//      Identifiers for Notebook tabs

        struct NoteBook_Tab_Id{
            uint SimControl_Tab_Id;
            uint ChemReactr_Views_Tab_Id;
            uint SimStats_Tab_Id;
            uint Help_Tab_Id;
            uint SysMon_Tab_Id;
        }  NoteBook_Tab_Ids;
//==========================================================================================================

    public:

    private:
        GUI_Presentation();   
        void Start();
        void Setup_Help_Tab();
        void Setup_Chem_And_Reactor_Tab();
        void Setup_Main_Menu();
        void Setup_Simulation_Control_Tab();
        void Setup_SimStats_Tab();
        void Setup_SysMon_Tab();

//      Functions to bind API to Gtk widgets

        void Bind_Drawing_Area_To_API(Widget_Ids::Drawing_Area_Id  Id, Drawing_Area*  Drawing_Area_Ptr);
        void Bind_ListStore_To_API(Widget_Ids::ListStore_Id  Id, 
                                   Glib::RefPtr<Gtk::ListStore> ListStore_RefPtr);
        void Bind_ListView_To_API(Widget_Ids::ListView_Id  Id, 
                                  Gtk::TreeView* ListView_Ptr);
        void Bind_TextBuffer_To_API(Widget_Ids::TextBuffer_Id  Id, 
                                    Glib::RefPtr<Gtk::TextBuffer> TextBuffer_RefPtr);


//      File Chooser functions

        bool Choose_File(string& PathName);

//      Notebook Functions

        string Get_Active_Page_Label();

//      ListView functions

        bool Get_Selected_Row(Widget_Ids::ListView_Id Id, 
                              list<string>& List_Of_Col_Texts, 
                              void*& Data_Ptr);
        bool Get_Selected_Row(Widget_Ids::ListView_Id Id, 
                              list<string>& List_Of_Col_Texts);
        void Update_Selected_Row(Widget_Ids::ListView_Id Id, 
                                 list<string>& List_Of_Col_Texts_Ptr, 
                                 void* Data_Ptr);
        bool Select_Row(Widget_Ids::ListView_Id Id, const string& Row_Tx);

        void Set_Selection_Mode(Widget_Ids::ListView_Id Id, 
                                GUI::ListView_SelMode   Mode);


//      ListStore functions

        void Add_Row(Widget_Ids::ListStore_Id  Id, 
                     list<string>& List_Of_Col_Texts, 
                     void*  Data_Ptr);
        void Add_Row(Widget_Ids::ListStore_Id  Id, 
                     list<string>& List_Of_Col_Texts);

        void Clear(Widget_Ids::ListStore_Id Id);

        void Get_Row_N_Col1(Widget_Ids::ListStore_Id Id, int RowNum, string& Row_Tx);

//      Drawing Area functions

        void Clear(Widget_Ids::Drawing_Area_Id Id);
        void Draw_Line(Widget_Ids::Drawing_Area_Id  Drawing_Area_Spec, 
                        int X1, int Y1, int X2, int Y2);   
        void Draw_Pixmap(Widget_Ids::Drawing_Area_Id   Drawing_Area_Spec);
        void Draw_Rectangle(Widget_Ids::Drawing_Area_Id   Drawing_Area_Spec,
                            bool Filled, int X, int  Y, int Width, int Height);
        void Get_Size(Widget_Ids::Drawing_Area_Id   Drawing_Area_Spec, int& Width, int&  Height);
        void Set_Foreground_Colour(Widget_Ids::Drawing_Area_Id  Drawing_Area_Spec,
                                   Colour_Id  Colour_Spec); 
        void Set_Foreground_Colour(Widget_Ids::Drawing_Area_Id  Drawing_Area_Spec,
                                   int Red, int Green, int Blue);
        void Text_Out(Widget_Ids::Drawing_Area_Id  Drawing_Area_Spec, int X, int Y, string Text);   

//      TextBuffer Functions

        void Add_Text(Widget_Ids::TextBuffer_Id Id, string&  Text);    
        void Clear(Widget_Ids::TextBuffer_Id Id); 
//----------------------------------------------------------------------------------------------------------------

//      Event Handlers

//      File Menu
        void Handle_File_Exit_Selection();
        void Handle_File_Load_Scen_Selection();
        void Handle_File_Close_Scen_Selection();

//      Simulation Menu
        void Handle_Simulation_Run_Selection();
        void Handle_Simulation_Stop_Selection();

//      ListView actions
        void Handle_MolType_ListView_Row_Select();
        void Handle_R_MolType_ListView_Row_Select();
        void Handle_SimStats_ListView_Series_Row_DblClick(const Gtk::TreeModel::Path &, Gtk::TreeViewColumn*);
//        void Handle_SimStats_ListView_SerList_Row_DblClick(const Gtk::TreeModel::Path &, Gtk::TreeViewColumn*);
        void Handle_SimStats_ListView_Groups_Row_DblClick(const Gtk::TreeModel::Path &, Gtk::TreeViewColumn*);
        void Handle_SimStats_ListView_SerList_Row_Select();

//      Notebook actions

        void Handle_Main_Notebook_Switch_Page(GtkNotebookPage* page, guint page_num);
//----------------------------------------------------------------------------------------------------------------

//      Helper classes for Chemistry and Reactor View Tab

//      Model Definition for the Chemistry ListViews

        class MolType_ListStore_Cols_Tp : public Gtk::TreeModel::ColumnRecord
        {
            friend class GUI_Presentation;  
            private:   
                MolType_ListStore_Cols_Tp()
                {   add(MolType_Id_Col); 
                    add(Mass_Col);
                    add(Potential_Energy_Col);
                    add(MolType_Ptr_Col);}      
                Gtk::TreeModelColumn<string> MolType_Id_Col;
                Gtk::TreeModelColumn<string> Mass_Col;
                Gtk::TreeModelColumn<string> Potential_Energy_Col;
                Gtk::TreeModelColumn<void*>  MolType_Ptr_Col;
        };
        MolType_ListStore_Cols_Tp MolType_ListStore_Cols;

        class IntType_ListStore_Cols_Tp : public Gtk::TreeModel::ColumnRecord
        {
            friend class GUI_Presentation;  
            private:   
                IntType_ListStore_Cols_Tp()
                {   add(IntType_Id_Col); 
                    add(Category_Col);
                    add(Heat_Col);
                    add(Fwd_Actn_Energy_Col);
                    add(Rev_Actn_Energy_Col);
                    add(Actv_Cmplx_Energy_Col);
                    add(Fwd_Rate_Const_Col);
                    add(Rev_Rate_Const_Col);
                    add(Reactant1_Col);
                    add(Reactant2_Col);
                    add(Product1_Col);
                    add(Product2_Col);
                    add(IntType_Ptr_Col);}      
                Gtk::TreeModelColumn<string> IntType_Id_Col;
                Gtk::TreeModelColumn<string> Category_Col;
                Gtk::TreeModelColumn<string> Heat_Col;
                Gtk::TreeModelColumn<string> Fwd_Actn_Energy_Col;
                Gtk::TreeModelColumn<string> Rev_Actn_Energy_Col;
                Gtk::TreeModelColumn<string> Actv_Cmplx_Energy_Col;
                Gtk::TreeModelColumn<string> Fwd_Rate_Const_Col;
                Gtk::TreeModelColumn<string> Rev_Rate_Const_Col;
                Gtk::TreeModelColumn<string> Reactant1_Col;
                Gtk::TreeModelColumn<string> Reactant2_Col;
                Gtk::TreeModelColumn<string> Product1_Col;
                Gtk::TreeModelColumn<string> Product2_Col;
                Gtk::TreeModelColumn<void*>  IntType_Ptr_Col;
        };
        IntType_ListStore_Cols_Tp IntType_ListStore_Cols;
//----------------------------------------------------------------------------------------------------------------

//      Model Definition for the Reactor ListViews

        class R_MolType_ListStore_Cols_Tp : public Gtk::TreeModel::ColumnRecord
        {
            friend class GUI_Presentation;  
            private:   
                R_MolType_ListStore_Cols_Tp()
                {   add(R_MolType_Id_Col); 
                    add(Concentration_Col);
                    add(R_MolType_Ptr_Col);}      
                Gtk::TreeModelColumn<string> R_MolType_Id_Col;
                Gtk::TreeModelColumn<string> Concentration_Col;
                Gtk::TreeModelColumn<void*>  R_MolType_Ptr_Col;
        };
        R_MolType_ListStore_Cols_Tp R_MolType_ListStore_Cols;

        class R_IntType_ListStore_Cols_Tp : public Gtk::TreeModel::ColumnRecord
        {
            friend class GUI_Presentation;  
            private:   
                R_IntType_ListStore_Cols_Tp()
                {   add(R_IntType_Id_Col); 
                    add(Category_Col);
                    add(Rate_Col);
                    add(Reactant1_Col);
                    add(Reactant2_Col);
                    add(Product1_Col);
                    add(Product2_Col);
                    add(R_IntType_Ptr_Col);}      
                Gtk::TreeModelColumn<string> R_IntType_Id_Col;
                Gtk::TreeModelColumn<string> Category_Col;
                Gtk::TreeModelColumn<string> Rate_Col;
                Gtk::TreeModelColumn<string> Reactant1_Col;
                Gtk::TreeModelColumn<string> Reactant2_Col;
                Gtk::TreeModelColumn<string> Product1_Col;
                Gtk::TreeModelColumn<string> Product2_Col;
                Gtk::TreeModelColumn<void*>  R_IntType_Ptr_Col;
        };
        R_IntType_ListStore_Cols_Tp R_IntType_ListStore_Cols;
//----------------------------------------------------------------------------------------------------------------

//      Helper classes for Simulation Statistics Tab
          
//      Model Definition for the Groups ListView

        class SimStats_ListStore_Groups_Cols_Tp : public Gtk::TreeModel::ColumnRecord
        {
            friend class GUI_Presentation;  
            private:   
                SimStats_ListStore_Groups_Cols_Tp()
                { add(Group_Name_Col); 
                    add(Display_Flag_Col);
                    add(Display_Group_Spec_Ptr_Col);}      
                Gtk::TreeModelColumn<string> Group_Name_Col;
                Gtk::TreeModelColumn<string> Display_Flag_Col;
                Gtk::TreeModelColumn<void*>  Display_Group_Spec_Ptr_Col;
        };
        SimStats_ListStore_Groups_Cols_Tp SimStats_ListStore_Groups_Cols;

//      Model Definition for the Series List ListView

        class SimStats_ListStore_SerList_Cols_Tp : public Gtk::TreeModel::ColumnRecord
        {
            friend class GUI_Presentation;  
            private:   
                SimStats_ListStore_SerList_Cols_Tp()
                { add(SerList_Name_Col); 
                  add(Display_Flag_Col);}
//                  add(Display_Group_Spec_Ptr_Col);}      
                Gtk::TreeModelColumn<string> SerList_Name_Col;
                Gtk::TreeModelColumn<string> Display_Flag_Col;
//                Gtk::TreeModelColumn<void*>  Display_Group_Spec_Ptr_Col;
        };
        SimStats_ListStore_SerList_Cols_Tp SimStats_ListStore_SerList_Cols;

//      Model Definition for the Series ListView

        class SimStats_ListStore_Series_Cols_Tp : public Gtk::TreeModel::ColumnRecord
        {
            friend class GUI_Presentation;  
            private:   
                SimStats_ListStore_Series_Cols_Tp()
                { add(Series_Name_Col); 
                    add(Plot_Flag_Col);
                    add(Indexed_Series_Ptr_Col);}      
                Gtk::TreeModelColumn<string> Series_Name_Col;
                Gtk::TreeModelColumn<string> Plot_Flag_Col;
                Gtk::TreeModelColumn<void*>  Indexed_Series_Ptr_Col;
        };
        SimStats_ListStore_Series_Cols_Tp SimStats_ListStore_Series_Cols;
//----------------------------------------------------------------------------------------------------------------

//      Helper class for Simulation Control Tab
          
//      Model Definition for the Actions ListView

        class Simulation_Actions_ListStore_Cols_Tp : public Gtk::TreeModel::ColumnRecord
        {
            friend class GUI_Presentation;  
            private:   
                Simulation_Actions_ListStore_Cols_Tp()
                   {add(Time_Col); 
                    add(Action_Col);
//                    add(Number_Col);
                    add(Parameters_Col);
                    add(Repeat_Col);
                   }      
                Gtk::TreeModelColumn<string> Time_Col;
                Gtk::TreeModelColumn<string> Action_Col;
//                Gtk::TreeModelColumn<string> Number_Col;
                Gtk::TreeModelColumn<string> Parameters_Col;
                Gtk::TreeModelColumn<string> Repeat_Col;
        };
        Simulation_Actions_ListStore_Cols_Tp Simulation_Actions_ListStore_Cols;

//----------------------------------------------------------------------------------------------------------------

//      Helper class for System Monitor Tab
          
//      Model Definition for the System Monitor ListView

        class SysMon_ListStore_Cols_Tp : public Gtk::TreeModel::ColumnRecord
        {
            friend class GUI_Presentation;  
            private:   
                SysMon_ListStore_Cols_Tp()
                   {add(Class_Col); 
                    add(Object_Count_Col);
                    add(Creations_Col);
                    add(Deletions_Col);
                   }      
                Gtk::TreeModelColumn<string> Class_Col;
                Gtk::TreeModelColumn<string> Object_Count_Col;
                Gtk::TreeModelColumn<string> Creations_Col;
                Gtk::TreeModelColumn<string> Deletions_Col;
        };
        SysMon_ListStore_Cols_Tp SysMon_ListStore_Cols;

};
#endif  //GUI_PRESENTATION_H
