/****************************************************************************
*
*     Module Filename:  GUI_Presentation_API.cpp
*     Module Name:      GUI_Presentation_API
*     Description:      API for the SimSoup Presentration Layer
*                       Isolates application code from the GUI toolkit by avoiding
*                       reference to the toolkit in the interface
*                      
*     Author:           Chris Gordon-Smith
*
*     Copyright (C) 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*     Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/
//  Comment out to disallow tracing this file
//#define  ENABLE_TRACE_FILE  

namespace  API {int  GUI_Presentation_API;} // Workaround for visibility in KDevelop explorer

#include    "globldef.h"
#include    "Library_Headers.h"

#include    <gtkmm.h>

//--------------  GUI Presentation Subsystem  --------------------------------------------------------------------
#include  "GUI_Presentation_API.h"
#include  "GUI_Presentation.h"
//----------------------------------------------------------------------------------------------------------------

GUI_Presentation*  GUI_Presentation_API::GUI_Presentation_Ptr = 0;
//----------------------------------------------------------------------------------------------------------------




void GUI_Presentation_API::Add_Row(Widget_Ids::ListStore_Id Id, list<string>& List_Of_Col_Texts)

//  Add a row to the specified ListStore. 

{
    Assert(Id < Widget_Ids::Num_ListStores, "GUI_Presentation_API::Add_Row(Widget_Ids::ListStore_Id Id, list<string>& List_Of_Col_Texts) - Invalid ListStore Id");
    GUI_Presentation_Ptr->Add_Row(Id, List_Of_Col_Texts);
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation_API::Add_Row(Widget_Ids::ListStore_Id Id, 
                                   list<string>& List_Of_Col_Texts, 
                                   void*  Data_Ptr)

//  Add a row to the specified ListStore, along with a pointer to associated data

{
    Assert(Id < Widget_Ids::Num_ListStores, "GUI_Presentation_API:Add_Row - Invalid ListStore Id");
    GUI_Presentation_Ptr->Add_Row(Id, List_Of_Col_Texts, Data_Ptr);
}
//----------------------------------------------------------------------------------------------------------------
void GUI_Presentation_API::Add_Text(Widget_Ids::TextBuffer_Id Id, string&  Text)

//  Add the specified text to the TextBuffer

{
    Assert(Id < Widget_Ids::Num_TextBuffers, "GUI_Presentation_API.Add_Text  - Invalid TextBuffer Id");
    GUI_Presentation_Ptr->Add_Text(Id, Text);
}
//----------------------------------------------------------------------------------------------------------------

bool GUI_Presentation_API::Choose_File(string&  PathName)

{
    return GUI_Presentation_Ptr->Choose_File(PathName);
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation_API::Clear(Widget_Ids::Drawing_Area_Id  Drawing_Area_Spec)

//  Clear the specified Drawing Area

{
    GUI_Presentation_Ptr->Clear(Drawing_Area_Spec);
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation_API::Clear(Widget_Ids::ListStore_Id Id)

//  Clear the specified ListStore

{
    GUI_Presentation_Ptr->Clear(Id);
}
//---------------------------------------------------------------------------------------------------------------

void GUI_Presentation_API::Clear(Widget_Ids::TextBuffer_Id Id)

//  Clear the specified Text Buffer

{
    GUI_Presentation_Ptr->Clear(Id);
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation_API::Get_Size(Widget_Ids::Drawing_Area_Id   Drawing_Area_Spec, int& Width, int&  Height)

//  Get the size of the specified Drawing Area

{
    GUI_Presentation_Ptr->Get_Size(Drawing_Area_Spec, Width, Height);
}
//----------------------------------------------------------------------------------------------------------------
/*
GUI_Presentation_API::SimSoup_Colour  GUI_Presentation_API::Laz_To_SimSoup_Colour(TColor  Col) 

// Temporary  Conversion routine

{
    if (Col == clBlue())

    THEN
        return Blue;
    ELSEIF (Col == clRed())
    THEN
        return Red;
    ELSEIF (Col == clGreen())
    THEN
        return Green;
    ELSEIF (Col == clBlack())
    THEN
        return Black;
    ELSEIF (Col == clWhite())
    THEN
        return White;
    ELSEIF (Col == clLime())
    THEN
        return LimeGreen;    
    ELSEIF (Col == clMaroon())
    THEN
        return Maroon;
    ELSE
        Assert (true == false, "GUI_Presentation_API::Laz_To_SimSoup_Colour - Invalid TColor specified");
        return Black; 
    ENDIF
} */
//----------------------------------------------------------------------------------------------------------------

void  GUI_Presentation_API::Start()

//  Create the Presentation Layer singleton object and start the presentation layer

{
    Assert(GUI_Presentation_Ptr  == 0,"GUI_Presentation_API::Make_UI -  Invalid call; GUI_Presentation has already been created");
    CREATE_OBJECT(GUI_Presentation, GUI_Presentation_Ptr = new GUI_Presentation);

    GUI_Presentation_Ptr->Start();
}
//----------------------------------------------------------------------------------------------------------------
/*void  GUI_Presentation_API::Run_UI()

//  Start the Gtk Event Loop 

{
    GUI_Presentation_Ptr->Run_UI();
} */
//-----------------------------------------------------------------------------------------------------------------

void  GUI_Presentation_API::Draw_Line(Widget_Ids::Drawing_Area_Id Drawing_Area_Spec, 
                                      int X1, int Y1, int X2, int Y2)

//  Draw a line on the specified Drawing Area using its current Foreground Colour

{        
    GUI_Presentation_Ptr->Draw_Line(Drawing_Area_Spec, X1, Y1, X2, Y2);
}
//----------------------------------------------------------------------------------------------------------------

void  GUI_Presentation_API::Draw_Pixmap(Widget_Ids::Drawing_Area_Id   Drawing_Area_Spec)

//  Display the Pixmap that has been built up by draw commands on the specified
//  Drawing Area to its visible area

{
    GUI_Presentation_Ptr->Draw_Pixmap(Drawing_Area_Spec);
}
//----------------------------------------------------------------------------------------------------------------

void  GUI_Presentation_API::Draw_Rectangle(Widget_Ids::Drawing_Area_Id   Drawing_Area_Spec,
                                           bool Filled, int X, int  Y, int Width, int Height)

//  Draw a rectangle on the specified Drawing Area

{
    GUI_Presentation_Ptr->Draw_Rectangle(Drawing_Area_Spec, Filled, X, Y, Width, Height);
}
//----------------------------------------------------------------------------------------------------------------

string GUI_Presentation_API::Get_Active_Page_Label()

//  Get the label of the active page. 

{
    return GUI_Presentation_Ptr->Get_Active_Page_Label();
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation_API::Get_Row_N_Col1(Widget_Ids::ListStore_Id Id, int RowNum, string& Row_Tx)

{
    GUI_Presentation_Ptr->Get_Row_N_Col1(Id, RowNum, Row_Tx);
}
//----------------------------------------------------------------------------------------------------------------

bool  GUI_Presentation_API::Get_Selected_Row(Widget_Ids::ListView_Id Id, 
                                             list<string>& List_Of_Col_Texts, 
                                             void*& Data_Ptr)

//  Get the ListStore row corresponding to the ListView selection, 
//  along with associated data

{
    return GUI_Presentation_Ptr->Get_Selected_Row(Id, List_Of_Col_Texts, Data_Ptr);
}
//----------------------------------------------------------------------------------------------------------------

bool  GUI_Presentation_API::Get_Selected_Row(Widget_Ids::ListView_Id Id, 
                                             list<string>& List_Of_Col_Texts)

//  Get the ListStore row corresponding to the ListView selection

{
    TRACE("GUI_Presentation_API::Get_Selected_Row(Widget_Ids::ListView_Id Id, list<string>& List_Of_Col_Texts)");
    return GUI_Presentation_Ptr->Get_Selected_Row(Id, List_Of_Col_Texts);
}
//----------------------------------------------------------------------------------------------------------------

bool GUI_Presentation_API::Select_Row(Widget_Ids::ListView_Id Id, const string& Row_Tx)

{
    return GUI_Presentation_Ptr->Select_Row(Id, Row_Tx);
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation_API::Set_Foreground_Colour(Widget_Ids::Drawing_Area_Id Drawing_Area_Spec, 
                                                 Colour_Id Colour_Spec)

//  Set the Foregroung colour for the specified Drawing Area

{
    GUI_Presentation_Ptr->Set_Foreground_Colour(Drawing_Area_Spec, Colour_Spec);
}
//------------------------------------------------------------------------------------------------------------------

void GUI_Presentation_API::Set_Foreground_Colour(Widget_Ids::Drawing_Area_Id Drawing_Area_Spec, 
                                                 int Red, int Green, int Blue)

//  Set the specified Drawing Area's Foreground Colour to the specified RGB combination

{
    GUI_Presentation_Ptr->Set_Foreground_Colour(Drawing_Area_Spec, Red, Green, Blue);
}
//-----------------------------------------------------------------------------------------------------------------

void GUI_Presentation_API::Set_Selection_Mode(Widget_Ids::ListView_Id Id,
                                              GUI::ListView_SelMode   Mode)

//  Set the selection mode for a Listview

{
    GUI_Presentation_Ptr->Set_Selection_Mode(Id, Mode);
}
//-----------------------------------------------------------------------------------------------------------------

void GUI_Presentation_API::Text_Out(Widget_Ids::Drawing_Area_Id Drawing_Area_Spec, 
                                    int X, int Y, string  Text)

//  Write the specified text to the specified location on the specified Drawing Area

{
    GUI_Presentation_Ptr->Text_Out(Drawing_Area_Spec, X, Y, Text);
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation_API::Update_Selected_Row(Widget_Ids::ListView_Id Id, 
                                               list<string>& List_Of_Col_Texts_Ptr, 
                                               void* Data_Ptr)

//  Update the ListStore row corresponding to the selection in the specified ListView

{
    GUI_Presentation_Ptr->Update_Selected_Row(Id, List_Of_Col_Texts_Ptr, Data_Ptr);
}


